@@ -1,3 +1,4 @@
+import numpy
 import re
 from typing import Union
 
@@ -102,17 +103,33 @@ def parse_brand(p: Product, html: str = None, selector: parsel.Selector = None):
 
 
 def parse_price(p: Product, html: str = None, selector: parsel.Selector = None):
-    s = selector.xpath("//span[@id='priceblock_ourprice']/text()").get()
-    s = s or selector.xpath("//span[@id='priceblock_saleprice']/text()").get()
-    s = s or selector.xpath("//span[@id='priceblock_dealprice']/text()").get()
-    s = s or selector.xpath("//div[@id='buyNew_noncbb']/span/text()").get()
-    s = s or selector.xpath("//*[@id='color_name_0_price']/span/text()[last()]").get()
-    if s is None:
-        p.price_min = None
-        p.price_max = None
-        return
+    def table() -> pandas.DataFrame:
+        t = pandas.DataFrame(index=("origin", "discount"), columns=('title', 'price'))
+        for i, tr in enumerate(selector.xpath("//div[@id='price']/table/tr")[:2]):
+            t.iat[i, 0] = tr.xpath("normalize-space(./td[1]/text())").get()
+            t.iat[i, 1] = tr.xpath("normalize-space(./td[2]/span[1]/text())").get()
+        return t
 
-    prices: Union[None, list] = parse_number(s, first=False)
+    def s1() -> str:
+        s = selector.xpath("//span[@id='priceblock_ourprice']/text()").get()
+        s = s or selector.xpath("//span[@id='priceblock_saleprice']/text()").get()
+        s = s or selector.xpath("//div[@id='buyNew_noncbb']/span/text()").get()
+        s = s or selector.xpath("//*[@id='color_name_0_price']/span/text()[last()]").get()
+        return s or ""
+
+    def s2() -> str:
+        t = table()
+        if numpy.all(t["price"] == ""):
+            return ""
+        if numpy.any(t["price"] == ""):
+            return t["price"][t["price"] != ""][0]
+        if t.at["discount", "title"] == "Deal of the Day:":
+            return t.at["origin", "price"]
+        else:
+            return t.at["discount", "price"]
+
+    prices_str = s1() or s2()
+    prices: Union[None, list] = parse_number(prices_str.replace(",", ""), first=False)
     if prices is None or len(prices) == 0:
         p.price_min = None
         p.price_max = None
@@ -158,19 +175,37 @@ def parse_stock(p: Product, html: str = None, selector: parsel.Selector = None):
 
 
 def parse_ship(p: Product, html: str = None, selector: parsel.Selector = None):
-    table = selector.xpath("//table[@id='tabular-buybox-container']").get()
-    if table is None:
+    def s1() -> str:
+        table = selector.xpath("//table[@id='tabular-buybox-container']").get()
+        if table is None:
+            return ""
+        table = pandas.read_html(table)
+        if len(table) <= 0:
+            return ""
+        table = table[0].set_index(0)
+        if "Ships from" not in table[1]:
+            return ""
+        s = table[1]["Ships from"]
+        return s
+
+    def s2() -> str:
+        ship = selector.xpath("normalize-space(//div[@id='sfsb_accordion_head']/div/div/span[2]/text())").get()
+        return ship or ""
+
+    def s3() -> str:
+        ship_sel = selector.xpath("normalize-space(string(//div[@id='merchant-info']))")
+        ship = ship_sel.re("Ships from and sold by (.+).$")
+        ship = ship or ship_sel.re("Sold by .+? and Fulfilled by (.+).$")
+        if len(ship) > 0:
+            return ship[0]
+        else:
+            return ""
+
+    ship_str = s1() or s2() or s3()
+    if ship_str != "":
+        p.ship = ship_str.lower()
+    else:
         p.ship = None
-        return
-    table = pandas.read_html(table)
-    if len(table) <= 0:
-        p.ship = None
-        return
-    table = table[0].set_index(0)
-    if "Ships from" not in table[1]:
-        p.ship = None
-        return
-    p.ship = table[1]["Ships from"]
 
 
 def parse_delivery(p: Product, html: str = None, selector: parsel.Selector = None):
